import commands
import logging
import sys
import argparse
import re
import platform

import parsing

# TODO detect primary interface by default route
# TODO check virtual interfaces (eth0:1) and funny iface names
# TODO check bonding/teaming
# TODO check IPless up interfaces

# Consts
BLACKLIST_IFACES_PREFIX = "lo"
INTERFACE_NAME_PREFIX_BY_PRIORITY = ["eth", "wlan", "en0", "en1"] # TODO move setting to OS-specific plugin
DEBUG_LEVEL = logging.WARNING # logging.DEBUG

logging.basicConfig(level=DEBUG_LEVEL)

# Globals
parse_ip_addr_cmd = None

def prioritize_ifaces(prio_list, keys):
    prioritized_list = []
    for prio in prio_list:
        cur_level = [k for k in keys if k.startswith(prio)]
        prioritized_list.extend(cur_level)

    # Add rest of the keys
    unprioritized_keys = [k for k in keys if k not in prioritized_list]

    prioritized_list.extend(unprioritized_keys)

    return prioritized_list

def parse_args(args):
    parse = argparse.ArgumentParser(description="myip: Simple tool to display the current IP")
    parse.add_argument('--all', const=True, action='store_const', dest="all_ips", help="show all IPs")
    parse.add_argument('interface', nargs='?', help="Show IP of a specific interface")
    config = parse.parse_args(args)

    return config

def generate_ip_list_ordered_by_iface(ip_hash, interface_list):
    ips = []
    for interface in interface_list:
        ips.append(ip_hash[interface])

    return ips

def get_ip_of_specific_interface(iface):
    return parse_ip_addr_cmd(iface)[iface]

def load_platform_support():
    # pylint: disable=W0603
    global parse_ip_addr_cmd
    os_name = platform.system().lower()

    if not parse_ip_addr_cmd:
        try:
            os_module = getattr(__import__("myip", fromlist=[os_name]), os_name)
            parse_ip_addr_cmd = os_module.parse_ip_addr_cmd
        except ImportError:
            logging.error("Failed loading platform %s support", os_name)

def get_ips(config):
    if config.interface: # interface param provided
        return [get_ip_of_specific_interface(config.interface)]

    ifaces_hash = parse_ip_addr_cmd()
    filtered_ifaces = [k for k in ifaces_hash.keys() if not k.startswith(BLACKLIST_IFACES_PREFIX)]

    ifaces_by_priority = prioritize_ifaces(INTERFACE_NAME_PREFIX_BY_PRIORITY, filtered_ifaces)
    if config.all_ips:
        # All interfaces
        ips = generate_ip_list_ordered_by_iface(ifaces_hash, ifaces_by_priority)
    else:
        # Primary interface only
        ips = generate_ip_list_ordered_by_iface(ifaces_hash, [ifaces_by_priority[0]])

    return ips
    
def main():
    load_platform_support()
    config = parse_args(sys.argv[1:])
    ips = get_ips(config)
    for ip in ips:
        print ip

if __name__ == "__main__":
    main()



import re
import logging
import commands
from . import parsing

def parse_ip_addr_cmd(specific_iface=None):
    cmdline = "ifconfig"
    if specific_iface:
        cmdline += " {iface}".format(iface=specific_iface)
    stat, out = commands.getstatusoutput(cmdline)
    if stat != 0:
        raise Exception("Command '{cmdline}' failed (rc={rc}): {out}".format(cmdline=cmdline, rc=stat, out=out))

    iface_ip_hash = {}
    for block in parsing.split_output_into_blocks(out):
        logging.debug("Working on block '%s'", block)
        match = re.search("^([^:]+):", block)
        if not match:
            logging.error("Bad block, skipping: '%s'", block)
            continue

        iface = match.groups()[0]
        if not "UP" in block:
            logging.info("'%s' is not UP, skipping", iface)
            continue
        right_side = block.partition("inet ")[2]
        if right_side == "":
            logging.info("No IP found for iface %s", iface)
            continue
        ip = right_side.partition(" ")[0]
        iface_ip_hash[iface] = ip
    return iface_ip_hash





def split_output_into_blocks(out):
    cur_entry = ""
    first_line = True
    for line in out.split("\n"):
        if first_line:
            first_line = False
            cur_entry += line + "\n"
            continue

        if not line.startswith(" ") and not line.startswith("\t"):
            yield cur_entry
            cur_entry = line + "\n"
        else:
            cur_entry += line + "\n"

    yield cur_entry # last but not least
 





import unittest
import myip_cmd
import linux
import darwin
from mock import patch, Mock


SAMPLE_OUTPUT_LINUX = """1: lo: <LOOPBACK,UP,LOWER_UP> mtu 16436 qdisc noqueue state UNKNOWN
    link/loopback 00:00:00:00:00:00 brd 00:00:00:00:00:00
    inet 127.0.0.1/8 scope host lo
    inet6 ::1/128 scope host
       valid_lft forever preferred_lft forever
2: eth0: <NO-CARRIER,BROADCAST,MULTICAST,UP> mtu 1500 qdisc pfifo_fast state DOWN qlen 1000
    link/ether 00:21:cc:b9:cb:d5 brd ff:ff:ff:ff:ff:ff
    inet 1.2.3.4/8 brd 1.255.255.255 scope global eth0
3: wlan0: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc mq state UP qlen 1000
    link/ether 10:0b:a9:81:ac:64 brd ff:ff:ff:ff:ff:ff
    inet 192.168.1.100/24 brd 192.168.1.255 scope global wlan0
    inet6 fe80::120b:a9ff:fe81:ac64/64 scope link
       valid_lft forever preferred_lft forever
4: virbr0: <NO-CARRIER,BROADCAST,MULTICAST,UP> mtu 1500 qdisc noqueue state DOWN
    link/ether 6a:5f:ce:b7:85:a7 brd ff:ff:ff:ff:ff:ff
    inet 192.168.122.1/24 brd 192.168.122.255 scope global virbr0"""

SAMPLE_OUTPUT_LINUX_NO_IP_ASSIGNED = """2: eth0: <NO-CARRIER,BROADCAST,MULTICAST,UP> mtu 1500 qdisc pfifo_fast state DOWN qlen 1000
    link/ether 00:21:cc:b9:cb:d5 brd ff:ff:ff:ff:ff:ff"""

SAMPLE_OUTPUT_DARWIN = """lo: flags=8049<UP,LOOPBACK,RUNNING,MULTICAST> mtu 16384
        inet 127.0.0.1 netmask 0xff000000 
        inet6 ::1 prefixlen 128 
        inet6 fe80::1%lo0 prefixlen 64 scopeid 0x1 
gif0: flags=8010<POINTOPOINT,MULTICAST> mtu 1280
stf0: flags=0<> mtu 1280
eth0: flags=8863<UP,BROADCAST,SMART,RUNNING,SIMPLEX,MULTICAST> mtu 1500
        inet6 fe80::214:51ff:fe68:77e0%en0 prefixlen 64 scopeid 0x4 
        inet 1.2.3.4 netmask 0xffffff00 broadcast 192.168.1.255
        ether 00:14:51:68:77:e0 
        media: autoselect (10baseT/UTP <half-duplex>) status: active
        supported media: none autoselect 10baseT/UTP <half-duplex> 10baseT/UTP <half-duplex,hw-loopback> 10baseT/UTP <full-duplex> 10baseT/UTP <full-duplex,hw-loopback> 10baseT/UTP <full-duplex,flow-control> 100baseTX <half-duplex> 100baseTX <half-duplex,hw-loopback> 100baseTX <full-duplex> 100baseTX <full-duplex,hw-loopback> 100baseTX <full-duplex,flow-control> 1000baseT <full-duplex> 1000baseT <full-duplex,hw-loopback> 1000baseT <full-duplex,flow-control>
wlan0: flags=8863<UP,BROADCAST,SMART,RUNNING,SIMPLEX,MULTICAST> mtu 1500
        inet6 fe80::214:51ff:fe68:77e0%en0 prefixlen 64 scopeid 0x4 
        inet 192.168.1.100 netmask 0xffffff00 broadcast 192.168.1.255
        ether 00:14:51:68:77:e0 
        media: autoselect (10baseT/UTP <half-duplex>) status: active
        supported media: none autoselect 10baseT/UTP <half-duplex> 10baseT/UTP <half-duplex,hw-loopback> 10baseT/UTP <full-duplex> 10baseT/UTP <full-duplex,hw-loopback> 10baseT/UTP <full-duplex,flow-control> 100baseTX <half-duplex> 100baseTX <half-duplex,hw-loopback> 100baseTX <full-duplex> 100baseTX <full-duplex,hw-loopback> 100baseTX <full-duplex,flow-control> 1000baseT <full-duplex> 1000baseT <full-duplex,hw-loopback> 1000baseT <full-duplex,flow-control>
en8: flags=8863<UP,BROADCAST,SMART,RUNNING,SIMPLEX,MULTICAST> mtu 1500
        ether 00:14:51:68:77:e1 
        media: autoselect (<unknown type>) status: inactive
        supported media: none autoselect 10baseT/UTP <half-duplex> 10baseT/UTP <half-duplex,hw-loopback> 10baseT/UTP <full-duplex> 10baseT/UTP <full-duplex,hw-loopback> 10baseT/UTP <full-duplex,flow-control> 100baseTX <half-duplex> 100baseTX <half-duplex,hw-loopback> 100baseTX <full-duplex> 100baseTX <full-duplex,hw-loopback> 100baseTX <full-duplex,flow-control> 1000baseT <full-duplex> 1000baseT <full-duplex,hw-loopback> 1000baseT <full-duplex,flow-control>
fw0: flags=8863<UP,BROADCAST,SMART,RUNNING,SIMPLEX,MULTICAST> mtu 4078
        lladdr 00:14:51:ff:fe:a8:a2:d2 
        media: autoselect <full-duplex> status: inactive
        supported media: autoselect <full-duplex>
virbr0: flags=8863<UP,BROADCAST,SMART,RUNNING,SIMPLEX,MULTICAST> mtu 1500
        inet6 fe80::214:51ff:fe68:77e0%en0 prefixlen 64 scopeid 0x4 
        inet 192.168.122.1 netmask 0xffffff00 broadcast 192.168.1.255
        ether 00:14:51:68:77:e0 
        media: autoselect (10baseT/UTP <half-duplex>) status: active
        supported media: none autoselect 10baseT/UTP <half-duplex> 10baseT/UTP <half-duplex,hw-loopback> 10baseT/UTP <full-duplex> 10baseT/UTP <full-duplex,hw-loopback> 10baseT/UTP <full-duplex,flow-control> 100baseTX <half-duplex> 100baseTX <half-duplex,hw-loopback> 100baseTX <full-duplex> 100baseTX <full-duplex,hw-loopback> 100baseTX <full-duplex,flow-control> 1000baseT <full-duplex> 1000baseT <full-duplex,hw-loopback> 1000baseT <full-duplex,flow-control>"""

SAMPLE_IP_HASH2 = {"wlan0": "192.168.1.100",
                "eth0": "1.2.3.4",
                "virbr0": "192.168.122.1",
                "lo": "127.0.0.1",
                }



class IpaddrLinuxParsingTests(unittest.TestCase):

    @patch("commands.getstatusoutput")
    def test_one_interface(self, mock_getoutput):
        mock_getoutput.return_value = (0, SAMPLE_OUTPUT_LINUX)
        self.assertEquals(SAMPLE_IP_HASH2, linux.parse_ip_addr_cmd(["wlan0"]))

    @patch("commands.getstatusoutput")
    def test_multiple_interfaces(self, mock_getoutput):
        mock_getoutput.return_value = (0, SAMPLE_OUTPUT_LINUX)
        self.assertEquals(SAMPLE_IP_HASH2, linux.parse_ip_addr_cmd([]))

    @patch("commands.getstatusoutput")
    def test_interface_with_no_ip_assigned(self, mock_getoutput):
        mock_getoutput.return_value = (0, SAMPLE_OUTPUT_LINUX_NO_IP_ASSIGNED)
        self.assertEquals({}, linux.parse_ip_addr_cmd([]))
    
class ifconfigDarwinParsingTests(unittest.TestCase):
    @patch("commands.getstatusoutput")
    def test_one_interface(self, mock_getoutput):
        mock_getoutput.return_value = (0, SAMPLE_OUTPUT_DARWIN)
        self.assertEquals(SAMPLE_IP_HASH2, darwin.parse_ip_addr_cmd(["wlan0"]))
 
    @patch("commands.getstatusoutput")
    def test_multiple_interfaces(self, mock_getoutput):
        mock_getoutput.return_value = (0, SAMPLE_OUTPUT_DARWIN)
        self.assertEquals(SAMPLE_IP_HASH2, darwin.parse_ip_addr_cmd(["wlan0"]))
    

class myipHighLevelTests(unittest.TestCase):
    def tearDown(self):
        reload(myip_cmd)

    def test_get_primary_ip(self):
        generate_ip_hash = Mock()
        generate_ip_hash.return_value = SAMPLE_IP_HASH2
        myip_cmd.parse_ip_addr_cmd = generate_ip_hash
        
        config = myip_cmd.parse_args([])
        ips = myip_cmd.get_ips(config)
        self.assertEquals(["1.2.3.4"], ips)

    def test_get_all_ips(self):
        generate_ip_hash = Mock()
        generate_ip_hash.return_value = SAMPLE_IP_HASH2
        myip_cmd.parse_ip_addr_cmd = generate_ip_hash
 
        config = myip_cmd.parse_args(["--all"])
        ips = myip_cmd.get_ips(config)
        self.assertEquals(["1.2.3.4", "192.168.1.100", "192.168.122.1"], ips)

    def test_specific_interface(self):
        generate_ip_hash = Mock()
        generate_ip_hash.return_value = SAMPLE_IP_HASH2
        myip_cmd.parse_ip_addr_cmd = generate_ip_hash
 
        config = myip_cmd.parse_args(["wlan0"])
        ips = myip_cmd.get_ips(config)
        self.assertEquals(["192.168.1.100"], ips)




#!/usr/bin/env python

from setuptools import setup, find_packages


setup(name = 'myip',
      version = '0.2',
      description = "Simply print the IP of the current host",
      author = 'Oren Held',
      author_email = 'oren@held.org.il',
      url = 'http://www.github.com/orenhe/myip',
      packages = find_packages(),
      license = 'MIT',
      entry_points = {
          'console_scripts': ['myip = myip.myip_cmd:main'],
      },
      install_requires = ["argparse"],
     )
